--[[
Notes by Andersen, Silvermoon

Summary:
Notes keeps track of notes in an easy-to-use GUI.
Raid leaders and raid assistants can print notes to
raid chat.  Notes marked as "important" will 
synchronize notes that are printed with anyone in 
the raid who is also using Notes via the CTRA channel.

Furthermore, any note can be executed as a LUA script
and called via a macro.  The macro script necessary to
execute the Notes script is easily visible.  This 
essentially makes a nearly unlimited macro length. 
Notes can also hook into the OnUpdate and OnEvent calls 
that WoW makes while playing for easy Add-On Development.

Thanks to Salamengel and Merphle (Author of LuaPad) 
for inspiration. Thanks to Sumul and Shinsplitter for 
suggestions.

use /notes to show the GUI

]]--

--[[
Table of Contents:
A) Variables
B) Notes Selection Functions
C) Notes Note Functions
D) LUA Functions
E) Sync Functions
F) UI Update Functions
G) UI Functions
H) Notes Sorting Functions
I) General Tools Functions
]]--

------------------------------------------
-- A) Variables

Notes_Notes = {}
local Notes_Undo = {}

Notes_LUAMode = false
local Notes_Selected = nil
local Notes_MultiSelect = {}
local Notes_LoadID = nil
local Notes_DescrLen = 25

local Notes_LastTime = nil
local Notes_PrintMinLength=10
local Notes_PrintMaxLength=90
local Notes_PrintMaxChunkLength=220

local Notes_PrintMinInterval=1
local Notes_PrintMaxInterval=6
local Notes_PrintInterval = Notes_PrintMaxInterval

local Notes_PrintCache = {}
local Notes_CacheIndex = 1
local Notes_Listening = false
local Notes_ListenStack = {}

local Notes_LUASetupNote = {"LUA - Setup","--Notes LUA Setup READ ME.\n\n--[[Notes can organize and execute multiple LUA scripts.  This setup file should help a new Notes user take advantage of Notes's LUA functionality.  Keep in mind that you must press the Run LUA button on this note when you start WoW and every time you make a change if you want to use the LUA functions defined here.  Unlock this note to modify it.\n\nEntering LUA mode: type /lua or /noteslua\nRunning LUA notes in macros: In LUA mode, any selected note will display text that you can copy and paste into a macro.  This uses the RunNote() function.]]--\n\n--[[OnEvent commands: Uncomment the following function and register some events to run OnEvent commands. ]]--\n\n\n--NotesEvents:RegisterEvent('CHAT_MSG_WHISPER')\n--Notes_OnEvent = function()\n--	doEventStuff()\n--end\n\n\n--[[OnUpdate commands: Uncomment the following function and register some events to run OnEvent commands. ]]--\n\n\n--Notes_OnUpdate = function()\n--	doUpdateStuff()\n--end\n\n"}

local Notes_ColorNormal = {1,.82,0}
local Notes_ColorImportant = {.7,1,.3}
local Notes_ColorLocked = {.52,.52,.52}
local Notes_ColorImportantLocked = {.4,.5,.7}

------------------------------------------
-- B) Notes Selection Functions
--[[
Notes_SelectByID - Select a note by id
Notes_NoteExists - Check to see if a note exists by id
Notes_SortAndReselect - Wrapper for Notes_SortNotes that fixes the selection after sorting
Notes_SortNotes - Alphabetize the notes
]]--

function Notes_SelectByID(id)
	for i=1,getn(Notes_Notes) do
		if Notes_Notes[i]["id"] == id then
			Notes_Selected = i
		end
	end
end

function Notes_NoteExists(id)
	for i=1,getn(Notes_Notes) do
		if tonumber(Notes_Notes[i]["id"]) == tonumber(id) then
			return i
		end
	end
	return false
end

function Notes_SortAndReselect()
	Notes_SortNotes()
	Notes_SelectByID(Notes_LoadID)
	
	if (Notes_Selected) then
		local tarscroll = (tonumber(Notes_Selected)-1)*16
		local minscroll = (0+FauxScrollFrame_GetOffset(NotesNoteScrollBar))*16
		local maxscroll = (4+FauxScrollFrame_GetOffset(NotesNoteScrollBar))*16
		local offset = nil
		
		if tarscroll < minscroll then
			offset = -2
		elseif tarscroll > maxscroll then
			offset = -4
		end		
		
		if offset then
			NotesNoteScrollBarScrollBar:SetValue((tonumber(Notes_Selected)+offset)*16)
		end
	end
end

function Notes_SortNotes()
	table.sort(Notes_Notes,Notes_HashNatTitleSort)
	Notes_ScrollBarUpdate()
end

------------------------------------------
-- C) Notes Note Functions
--[[
Notes_NewNote - Execute the selected LUA script note
Notes_SaveNote - wrapper for Notes_RunNote
Notes_CopySelected - Execute a LUA script note by passing a note id
Notes_LoadSelected - Add the default LUA setup note
Notes_DeleteAndRebuild - Delete the selected note (and rebuild the table with new, consecutive ids)
Notes_RevertSelected - Revert changes to the selected note to the last time the UI was shown
Notes_PrepUndo - Populate the undo table with the correct data.
Notes_ImportantSelected - Mark the selected note as important for syncing purposes
Notes_LockedSelected - Lock the selected note
]]--

function Notes_NewNote(title,text,important,locked,id,overwrite)
	if title == nil or title == "" then title = Notes_FindNextInSequence("New Note #1") end
	if text == nil then text = "" end
	if id == nil then id = time()..random(1000) end
	if important == nil then important = false end
	if locked == nil then locked = false end
	if rewrite == nil then rewrite = false end

	local newNote = {}
	newNote["title"] = title
	newNote["text"] = text
	newNote["id"] = id
	newNote["important"] = important
	newNote["locked"] = locked
	
	local oldidx = Notes_NoteExists(id)
	if oldidx then
		if not overwrite then return
		else
			if not Notes_Notes[oldidx]["locked"] then
				Notes_Notes[oldidx]["title"] = newNote["title"]
				Notes_Notes[oldidx]["text"] = newNote["text"]
				Notes_Notes[oldidx]["important"] = newNote["important"]
				Notes_Notes[oldidx]["locked"] = newNote["locked"]
			else
				Notes_ErrorPrint("NOTES ERROR! TRIED TO UPDATE ["..Notes_ListenStack["title"].."] BUT IT IS LOCKED!")
			end
		end
	else
		table.insert(Notes_Notes,newNote)
	end

	Notes_Undo[newNote["id"]] = newNote
	Notes_Undo[newNote["id"]]["undo"] = false
		
	if not overwrite then 
		Notes_SelectByID(newNote["id"])
		Notes_LoadSelected()
	end
	
	Notes_SortAndReselect()
	Notes_ScrollBarUpdate()
end

function Notes_SaveNote()
	local title = NotesTitleEditBox:GetText()
	local text = NotesTextEditBox:GetText()
	
	if not Notes_Selected then
		if strlen(title) > 0 or strlen(text) > 0 then 
			Notes_NewNote(title,text)
			return
			
		elseif strlen(title) == 0 and strlen(text) == 0 then
			return
		end
	end
	
	if Notes_Notes[Notes_Selected]["locked"] then
		NotesTitleEditBox:SetText(Notes_Notes[Notes_Selected]["title"])
		NotesTextEditBox:SetText(Notes_Notes[Notes_Selected]["text"])
		NotesImportantCheckButton:SetChecked(Notes_Notes[Notes_Selected]["important"])
		return
	end
	
	Notes_Notes[Notes_Selected]["title"] = title	
	Notes_Notes[Notes_Selected]["text"] = text
	
	if Notes_Undo[Notes_LoadID]["undo"] == false then	
	   if(strlen(Notes_Undo[Notes_LoadID]["title"]) ~= strlen(Notes_Notes[Notes_Selected]["title"]) or
	    strlen(Notes_Undo[Notes_LoadID]["text"]) ~= strlen(Notes_Notes[Notes_Selected]["text"])) then
	    	Notes_Undo[Notes_LoadID]["undo"] = true
	end end
	
	Notes_ScrollBarUpdate()
end

function Notes_CopySelected()
	local copyNote = {}
	copyNote["title"] = Notes_FindNextInSequence(Notes_Notes[Notes_Selected]["title"].." Copy #1")
	copyNote["text"] = Notes_Notes[Notes_Selected]["text"]
	copyNote["id"] = time()..random(1000)
	copyNote["important"] = Notes_Notes[Notes_Selected]["important"]
	copyNote["locked"] = Notes_Notes[Notes_Selected]["locked"]
	
	for i=1,getn(Notes_Notes) do
		if Notes_Notes[i]["id"] == copyNote["id"] then
			return
		end
	end

	table.insert(Notes_Notes,copyNote)

	Notes_Undo[copyNote["id"]] = copyNote
	Notes_Undo[copyNote["id"]]["undo"] = false	
	
	Notes_SortAndReselect()
	Notes_ScrollBarUpdate()
end

function Notes_LoadSelected(clear)
	if clear == nil then
		NotesTextEditBox:SetText(Notes_Notes[Notes_Selected]["text"])
		NotesTitleEditBox:SetText(Notes_Notes[Notes_Selected]["title"])
		Notes_LoadID = Notes_Notes[Notes_Selected]["id"]
	else
		Notes_ToggleSelected(nil)
		NotesTextEditBox:SetText("")
		NotesTitleEditBox:SetText("")
	end
	
	Notes_SortAndReselect()
	Notes_ScrollBarUpdate()
end

function Notes_DeleteAndRebuild()
	if not Notes_Selected or Notes_Notes[Notes_Selected]["locked"] then return end

	local newNotes_Notes = {}
	for i=1,getn(Notes_Notes) do
		if i ~= Notes_Selected then
			table.insert(newNotes_Notes,Notes_Notes[i])
		end	
	end
	Notes_Notes = newNotes_Notes
	
	Notes_LoadSelected(1)
	Notes_ScrollBarUpdate()
end

function Notes_RevertSelected()
	if Notes_Selected and Notes_Undo[Notes_LoadID]["undo"] == true then
		Notes_Notes[Notes_Selected]["title"] = Notes_Undo[Notes_LoadID]["title"]
		Notes_Notes[Notes_Selected]["text"] = Notes_Undo[Notes_LoadID]["text"]
		Notes_Notes[Notes_Selected]["important"] = Notes_Undo[Notes_LoadID]["important"]
		Notes_Notes[Notes_Selected]["locked"] = Notes_Undo[Notes_LoadID]["locked"]
		Notes_Undo[Notes_LoadID]["undo"] = false
		
		Notes_SortAndReselect()
		Notes_ScrollBarUpdate()
	end
end

function Notes_PrepUndo()
	for i=1,getn(Notes_Notes) do
		Notes_Undo[Notes_Notes[i]["id"]]={}
		Notes_Undo[Notes_Notes[i]["id"]]["id"] = Notes_Notes[i]["id"]
		Notes_Undo[Notes_Notes[i]["id"]]["title"] = Notes_Notes[i]["title"]
		Notes_Undo[Notes_Notes[i]["id"]]["text"] = Notes_Notes[i]["text"]
		Notes_Undo[Notes_Notes[i]["id"]]["important"] = Notes_Notes[i]["important"]
		Notes_Undo[Notes_Notes[i]["id"]]["locked"] = Notes_Notes[i]["locked"]
		Notes_Undo[Notes_Notes[i]["id"]]["undo"] = false		
	end
end

function Notes_ImportantSelected()
	if Notes_Notes[Notes_Selected]["locked"] then
		NotesImportantCheckButton:SetChecked(Notes_Notes[Notes_Selected]["important"])
	else
		Notes_Notes[Notes_Selected]["important"] = NotesImportantCheckButton:GetChecked()
	end
	
	Notes_ScrollBarUpdate()
end

function Notes_LockedSelected()
	Notes_Notes[Notes_Selected]["locked"] = NotesLockedCheckButton:GetChecked()
	
	Notes_ScrollBarUpdate()
end

------------------------------------------
-- D) LUA Functions
--[[
Notes_ExecuteSelected - Execute the selected LUA script note
RunNote - wrapper for Notes_RunNote
Notes_RunNote - Execute a LUA script note by passing a note id
Notes_AddLUASetupNote - Add the default LUA setup note
]]--

function Notes_ExecuteSelected()
	RunScript(Notes_Notes[Notes_Selected]["text"])
end

function RunNote(noteid)
	return Notes_RunNote(noteid)
end

function Notes_RunNote(noteid)
	local idx = nil
	for i=1,getn(Notes_Notes) do
		if tonumber(Notes_Notes[i]["id"]) == noteid then
			idx = i
			break
		end
	end
	if idx ~= nil then
		RunScript(Notes_Notes[idx]["text"])
	end
end

function Notes_AddLUASetupNote()
	Notes_NewNote(Notes_LUASetupNote[1],Notes_LUASetupNote[2],false,1,"1")
end

------------------------------------------
-- E) Sync Functions
--[[
Notes_PrintSelected - Load a note into the print cache for later parsing
Notes_ParseCache - Stagger printing from the messages in the print cache
Notes_ListenEvent - Listen to raid chat for synchronizing notes
]]--

function Notes_PrintSelected()
	if (Notes_Selected and Notes_InRaid() and (IsRaidLeader() or IsRaidOfficer())) or true then
		local note = Notes_Notes[Notes_Selected]["text"]--string.gsub(Notes_Notes[Notes_Selected]["text"],"\n+","\n")
		local notelength = strlen(note)
		
		local notebits = Notes_Split(note,"\n")
		local startloc = getn(Notes_PrintCache)+1
		
		if Notes_Notes[Notes_Selected]["important"] then
			local impmsg = "<NOTE::"..Notes_Notes[Notes_Selected]["title"].."::"..Notes_Notes[Notes_Selected]["id"].."::"..notelength..">"
			table.insert(Notes_PrintCache,impmsg)
		end
		
		for i=1,getn(notebits) do
			if notebits[i] == "" then 
				notebits[i] = " "
				notelength = notelength + 1
				if Notes_Notes[Notes_Selected]["important"] then
					Notes_PrintCache[startloc] = "<NOTE::"..Notes_Notes[Notes_Selected]["title"].."::"..Notes_Notes[Notes_Selected]["id"].."::"..notelength..">"
				end
			end
			
			if strlen(notebits[i]) <= Notes_PrintMaxChunkLength then
				table.insert(Notes_PrintCache,notebits[i])
			else
				local notebitswords = Notes_Split(notebits[i]," ")
				local notebitsword = ""
				
				for j=1,getn(notebitswords) do
					local testword = notebitsword.." "..notebitswords[j]
					if strlen(testword) < Notes_PrintMaxChunkLength then
						notebitsword = testword
					else
						table.insert(Notes_PrintCache,notebitsword)
						notebitsword = notebitswords[j]

						notelength = notelength + 1
						if Notes_Notes[Notes_Selected]["important"] then
							Notes_PrintCache[startloc] = "<NOTE::"..Notes_Notes[Notes_Selected]["title"].."::"..Notes_Notes[Notes_Selected]["id"].."::"..notelength..">"
						end
					end
				end
				table.insert(Notes_PrintCache,notebitsword)
			end
		end
		
		if Notes_Notes[Notes_Selected]["important"] then
			table.insert(Notes_PrintCache,"<NOTE::END>")
		end
	end
end

function Notes_ParseCache()
	if getn(Notes_PrintCache) > 0 then
	
		if not Notes_LastTime then
			Notes_LastTime = 0
		end
		
		local curtime = time()
		if curtime-Notes_LastTime > Notes_PrintInterval then
			if Notes_PrintCache[Notes_CacheIndex] ~= nil then
				Notes_RaidPrint(Notes_PrintCache[Notes_CacheIndex])
				
				if strsub(Notes_PrintCache[Notes_CacheIndex],1,7) == "<NOTE::" then
					Notes_PrintInterval=1
				else
					Notes_PrintInterval=((Notes_PrintMaxInterval-Notes_PrintMinInterval)*
					  (strlen(Notes_PrintCache[Notes_CacheIndex])-Notes_PrintMinLength)/
					  (Notes_PrintMaxLength-Notes_PrintMinLength))+Notes_PrintMinInterval
				end
				  
				if Notes_PrintInterval < Notes_PrintMinInterval then Notes_PrintInterval = Notes_PrintMinInterval end
				if Notes_PrintInterval > Notes_PrintMaxInterval then Notes_PrintInterval = Notes_PrintMaxInterval end

				Notes_CacheIndex = Notes_CacheIndex + 1
				Notes_LastTime = curtime				
			else
				Notes_PrintCache = {}
				Notes_CacheIndex = 1
				Notes_LastTime = nil
				Notes_PrintInterval = Notes_PrintMaxInterval
			end
		end
	end
end

function Notes_ListenEvent()
	if event == "RAID_ROSTER_UPDATE" then
		Notes_SetButtonState()
	end

	if Notes_Listening and Notes_Listening == arg2 and (event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER") then
		Notes_ListenStack["text"] = Notes_ListenStack["text"]..arg1.."\n"
		
	elseif (event == "CHAT_MSG_RAID" or event == "CHAT_MSG_RAID_LEADER") or 
	   (event == "CHAT_MSG_CHANNEL" and CT_RA_Channel ~= nil and strlower(arg9) == strlower(CT_RA_Channel)) then
		if strsub(arg1,1,7) == "<NOTE::" then 
			parsebits = Notes_Split(strsub(arg1,0,-2),"::")
			if Notes_Listening and parsebits[2] == "END" then
				if tonumber(strlen(strsub(Notes_ListenStack["text"],0,-2))) == tonumber(Notes_ListenStack["length"]) then
					Notes_NewNote(Notes_ListenStack["title"],Notes_ListenStack["text"],1,false,Notes_ListenStack["id"],true)
					Notes_SuccessPrint("SYNC SUCCESSFUL FOR ["..Notes_ListenStack["title"].."]!")
					
					Notes_SortAndReselect()
					Notes_ScrollBarUpdate()
				else
					Notes_ErrorPrint("NOTES ERROR! SYNC FAILED FOR ["..Notes_ListenStack["title"].."]!")
				end
				Notes_ListenStack = {}
				Notes_Listening = false
				
			elseif not Notes_Listening and arg2 ~= UnitName("player") then
			
				Notes_ListenStack["title"] = parsebits[2]
				Notes_ListenStack["id"] = parsebits[3]
				Notes_ListenStack["length"] = parsebits[4]
				Notes_ListenStack["text"] = ""
				Notes_Listening = arg2
			end
		end
		
		--[[if Notes_Listening and tonumber(strlen(strsub(Notes_ListenStack["text"],0,-2))) > tonumber(Notes_ListenStack["length"]) then		
				Notes_ListenStack = {}
				Notes_Listening = false
		end]]--
	end
end

------------------------------------------
-- F) UI Update Functions
--[[
Notes_ToggleSelected - Register that we've selected a new note
Notes_SetButtonState - Split a string into an array via a delimeter
Notes_GetColor - Get the proper color for a note's title
Notes_ScrollBarUpdate - Update the graphics for the notes navigation system
]]--

function Notes_ToggleSelected(button)
	if button ~= nil then
		local num = button
		if getglobal(button) then
			num = tonumber(strsub(button,-1))
		end
	
		Notes_Selected = num + FauxScrollFrame_GetOffset(NotesNoteScrollBar)
	else
		Notes_Selected = nil
		Notes_LoadID = nil
	end
	Notes_ScrollBarUpdate()
end

function Notes_SetButtonState()
	if Notes_Selected then
		if Notes_Notes[Notes_Selected]["locked"] then NotesDeleteButton:Disable()
		else NotesDeleteButton:Enable() end		
		
		NotesDuplicateButton:Enable()	
		NotesExecuteButton:Enable()
		NotesMacroEditBox:SetText("/script RunNote("..Notes_Notes[Notes_Selected]["id"]..")")
		NotesMacroEditBox:Show()		
		NotesImportantCheckButtonText:SetTextColor(Notes_ColorImportant[1],Notes_ColorImportant[2],Notes_ColorImportant[3])
		
		if Notes_Notes[Notes_Selected]["locked"] then 
			NotesImportantCheckButton:Disable()
			NotesImportantCheckButtonText:SetTextColor(Notes_ColorImportantLocked[1],Notes_ColorImportantLocked[2],Notes_ColorImportantLocked[3])
		else 
			NotesImportantCheckButton:Enable() 
		end		
		
		NotesImportantCheckButton:SetChecked(Notes_Notes[Notes_Selected]["important"])
		NotesLockedCheckButtonText:SetTextColor(Notes_ColorNormal[1],Notes_ColorNormal[2],Notes_ColorNormal[3])
		NotesLockedCheckButton:Enable()
		NotesLockedCheckButton:SetChecked(Notes_Notes[Notes_Selected]["locked"])
		
		if (Notes_InRaid() and (IsRaidLeader() or IsRaidOfficer())) then
			NotesPrintButton:Enable()
		else
			NotesPrintButton:Disable()
		end
	else
		NotesDeleteButton:Disable()
		NotesDuplicateButton:Disable()	
		NotesExecuteButton:Disable()
		NotesMacroEditBox:SetText("")
		NotesMacroEditBox:Hide()
		NotesImportantCheckButton:Disable()
		NotesImportantCheckButton:SetChecked(false)
		NotesImportantCheckButtonText:SetTextColor(Notes_ColorLocked[1],Notes_ColorLocked[2],Notes_ColorLocked[3])
		NotesLockedCheckButton:Disable()
		NotesLockedCheckButton:SetChecked(false)
		NotesLockedCheckButtonText:SetTextColor(Notes_ColorLocked[1],Notes_ColorLocked[2],Notes_ColorLocked[3])	
		NotesPrintButton:Disable()
	end
	
	if Notes_LUAMode then
		NotesLUACheckButton:SetChecked(1)
		ShowUIPanel(NotesLuaFrame);
	else
		NotesLUACheckButton:SetChecked(nil)
		HideUIPanel(NotesLuaFrame);
	end
	
	if Notes_LoadID and Notes_Undo[Notes_LoadID]["undo"] == true and 
	   Notes_Selected and not Notes_Notes[Notes_Selected]["locked"] then
		NotesUndoButton:Enable()
	else
		NotesUndoButton:Disable()
	end
	
	if Notes_NoteExists(1) then NotesLUASetupButton:Disable()
	else NotesLUASetupButton:Enable() end
end

function Notes_GetColor(index)
	if type(index) == "string" then
		local num = index
		if getglobal(index) then
			num = tonumber(strsub(index,-1))
			index = num + FauxScrollFrame_GetOffset(NotesNoteScrollBar)		 
		end		
	end
	
	local color = Notes_ColorNormal
	if Notes_Notes[index] then
		if Notes_Notes[index]["important"] then
			color = Notes_ColorImportant
		end
		if Notes_Notes[index]["locked"] then
			color = Notes_ColorLocked
		end
		if Notes_Notes[index]["important"] and Notes_Notes[index]["locked"] then
			color = Notes_ColorImportantLocked
		end
	end
	return color
end

function Notes_ScrollBarUpdate()
	local line; -- 1 through 5 of our window to scroll
  	local lineplusoffset; -- an index into our data calculated from the scroll offset
  	FauxScrollFrame_Update(NotesNoteScrollBar,getn(Notes_Notes),5,16);
  	for line=1,5 do
    		lineplusoffset = line + FauxScrollFrame_GetOffset(NotesNoteScrollBar);
    		if lineplusoffset <= getn(Notes_Notes) then
			local title = Notes_Notes[lineplusoffset]["title"]
			local descr = Notes_Notes[lineplusoffset]["text"]
			
			local newdescr = ""
			if descr ~= nil then
				newdescr = strsub(descr,0,Notes_DescrLen)
			  	if strlen(descr) > Notes_DescrLen then
					newdescr = newdescr.."..."
				end
			end
			
			if Notes_Selected == lineplusoffset then
				getglobal("NotesNote"..line.."NTex"):SetAlpha(1)
			else
				getglobal("NotesNote"..line.."NTex"):SetAlpha(0)
			end
			
			local textcolor = Notes_GetColor(lineplusoffset)
			
			getglobal("NotesNote"..line.."_Text"):SetTextColor(textcolor[1],textcolor[2],textcolor[3]);
      			getglobal("NotesNote"..line.."_Text"):SetText(title);
      			getglobal("NotesNote"..line.."_Description"):SetText(string.gsub(gsub(newdescr,"\n"," "),"%s+"," "))
      			getglobal("NotesNote"..line):Show();
    		else
      			getglobal("NotesNote"..line):Hide();
    		end
  	end
	Notes_SetButtonState()
end

------------------------------------------
-- G) UI Functions
--[[
Notes_OnLoad - Register Notes with the system
Notes_OnUpdate - This function's body is defined dynamically
Notes_OnEvent - This function's body is defined dynamically
Notes_UIToggle - Hide or show Notes
Notes_HideNotes - Clear the selection when Notes is hidden
Notes_LUAUIToggle - Hide or show the LUA window
]]--

function Notes_OnLoad()
	SlashCmdList["NOTES"] = Notes_UIToggle;
	SLASH_NOTES1 = "/notes";
	SLASH_NOTES2 = "/note";

	this:RegisterEvent('CHAT_MSG_RAID');
	this:RegisterEvent('CHAT_MSG_RAID_LEADER');
	this:RegisterEvent('CHAT_MSG_CHANNEL');
	this:RegisterEvent('RAID_ROSTER_UPDATE');	

	Notes_Print(Notes_Notes)
	Notes_SetButtonState()
end

function Notes_OnUpdate()
-- This function's body is defined dynamically
end

function Notes_OnEvent()
-- This function's body is defined dynamically
end

function Notes_UIToggle(msg)
	if( NotesFrame:IsVisible() ) then
		Notes_HideNotes();
	else
		ShowUIPanel(NotesFrame);
	end
end

function Notes_HideNotes()
	Notes_LoadSelected(1)
	NotesFrame:Hide()
end

function Notes_LUAUIToggle(mode)
	if mode==nil then mode = "toggle" end
	
	if mode == "hide" or (mode=="toggle" and NotesLuaFrame:IsVisible()) then
		Notes_LUAMode = false
		Notes_SetButtonState()
		
	elseif mode == "show" or mode=="toggle" then
		Notes_LUAMode = true
		Notes_SetButtonState()
	end
end

------------------------------------------
-- H) Notes Sorting Functions
--[[
Notes_HashNatSort - Natural sorting for numbers that appear after the last # of the value
Notes_HashNatTitleSort - Natural sorting for numbers that appear after the last # of the value's title
Notes_FindNextInSequence - Find the next value in a sequence
Notes_FindLast - Reverse string search
]]--

function Notes_HashNatSort(a,b)
	local hashloca = Notes_FindLast(a,"#")
	local hashlocb = Notes_FindLast(b,"#")

	if not hashloca or not hashlocb then return a < b end

	local astr = strsub(a,0,hashloca)
	local bstr = strsub(b,0,hashlocb)

	if astr ~= bstr then return a < b end

	local anum = tonumber(strsub(a,hashloca+1))
	local bnum = tonumber(strsub(b,hashlocb+1))

	if not anum or not bnum then return a < b end

	return anum < bnum
end

function Notes_HashNatTitleSort(a,b)
	if a["title"] == b["title"] then return a["id"] < b["id"] end

	local hashloca = Notes_FindLast(a["title"],"#")
	local hashlocb = Notes_FindLast(b["title"],"#")

	if not hashloca or not hashlocb then return a["title"] < b["title"] end

	local astr = strsub(a["title"],0,hashloca)
	local bstr = strsub(b["title"],0,hashlocb)

	if astr ~= bstr then return a["title"] < b["title"] end

	local anum = tonumber(strsub(a["title"],hashloca+1))
	local bnum = tonumber(strsub(b["title"],hashlocb+1))

	if not anum or not bnum then return a["title"] < b["title"] end

	return anum < bnum
end

function Notes_FindNextInSequence(title) -- requires to be in the format: "TITLE #<num>"
	local titles = {}	
	local hashloc = Notes_FindLast(title,"#")
		
	local prefix = strsub(title,0,hashloc)
	local num = tonumber(strsub(title,hashloc+1))
		
	title = prefix..num
	
	for i=1,getn(Notes_Notes) do
		table.insert(titles,Notes_Notes[i]["title"])
	end
	table.sort(titles,Notes_HashNatSort)

	for i=1,getn(titles) do
		if titles[i] == title then
			num = num + 1
			title = prefix..num
		end
	end
	return title
end

function Notes_FindLast(haystack,needle)
	local loc = string.find(haystack,needle)
	i=1
	while loc do
		local testloc = string.find(haystack,needle,loc+1)
		if testloc then loc = testloc 
		else break end
	end
	return loc
end

------------------------------------------
-- I) General Tools Functions
--[[
Notes_Split - Split a string into an array via a delimeter
Notes_RaidPrint - Do your best to print to raid chat
Notes_Print - General purpose print function.  Better than the default.
Notes_SuccessPrint - Print a success message
Notes_ErrorPrint - Print an error message
Notes_InParty - Am I in a party?
Notes_InRaid - Am I in a raid?
]]--


function Notes_Split(text, delimiter)
	local list = {};
	local pos = 1;
	if strfind("", delimiter, 1) then -- this would result in endless loops
		return text;
	end
	while 1 do
		local first, last = strfind(text, delimiter, pos);
		if first then
			tinsert(list, strsub(text, pos, first-1));
			pos = last+1;
		else
			tinsert(list, strsub(text, pos));
			break;
		end
	end
	return list;
end

function Notes_RaidPrint (msg)
	local index = nil
	if CT_RA_Channel ~= nil and strsub(msg,1,7) == "<NOTE::" then
		index = GetChannelName(CT_RA_Channel);
	end
	
	if index ~= nil and index ~= 0 then 
		SendChatMessage(msg,"CHANNEL",nil,index);		

	elseif Notes_InRaid() then
		SendChatMessage(msg, RAID);
		
        elseif Notes_InParty() then
		SendChatMessage(msg, PARTY);
		
	else
        	Notes_Print(msg);
        end
end

function Notes_Print(msg,prefix)
	if prefix == nil then prefix = "" end

	if msg == nil then 
		DEFAULT_CHAT_FRAME:AddMessage(prefix.."nil");
	
	elseif type(msg) == "number" then
		DEFAULT_CHAT_FRAME:AddMessage(prefix..msg);
		
	elseif type(msg) == "string" then
		DEFAULT_CHAT_FRAME:AddMessage(prefix..msg);
		
	elseif type(msg) == "boolean" and msg==false then
		DEFAULT_CHAT_FRAME:AddMessage(prefix.."false");
		
	elseif type(msg) == "boolean" and msg==true then
		DEFAULT_CHAT_FRAME:AddMessage(prefix.."true");
	
	elseif type(msg) == "table" then
		for index,item in msg do
			if type(item) == "table" then
				DEFAULT_CHAT_FRAME:AddMessage(prefix..index.."  --TABLE--");
				Notes_Print(item,prefix..">  ");
			else
				Notes_Print(item,prefix..index.."  ");
			end
		end
	end
end

function Notes_SuccessPrint(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg,.6,1,.3);
end

function Notes_ErrorPrint(msg)
	DEFAULT_CHAT_FRAME:AddMessage(msg,1,0,0);
end

function Notes_InParty()
	return GetPartyMember(1);
end

function Notes_InRaid()
	return UnitInRaid("player");
end
